@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,17 +16,20 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core.client_options import ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
-OptionalRetry = Union[retries.Retry, object]
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
 
 from google.cloud.debugger_v2.types import data
 from google.cloud.debugger_v2.types import debugger
@@ -108,6 +111,42 @@ class Debugger2AsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return Debugger2Client.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> Debugger2Transport:
         """Returns the transport used by the client instance.
@@ -181,6 +220,26 @@ class Debugger2AsyncClient:
     ) -> debugger.SetBreakpointResponse:
         r"""Sets the breakpoint to the debuggee.
 
+        .. code-block:: python
+
+            from google.cloud import debugger_v2
+
+            def sample_set_breakpoint():
+                # Create a client
+                client = debugger_v2.Debugger2Client()
+
+                # Initialize request argument(s)
+                request = debugger_v2.SetBreakpointRequest(
+                    debuggee_id="debuggee_id_value",
+                    client_version="client_version_value",
+                )
+
+                # Make the request
+                response = client.set_breakpoint(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.debugger_v2.types.SetBreakpointRequest, dict]):
                 The request object. Request to set a breakpoint
@@ -217,7 +276,7 @@ class Debugger2AsyncClient:
                 Response for setting a breakpoint.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([debuggee_id, breakpoint_, client_version])
         if request is not None and has_flattened_params:
@@ -264,6 +323,27 @@ class Debugger2AsyncClient:
     ) -> debugger.GetBreakpointResponse:
         r"""Gets breakpoint information.
 
+        .. code-block:: python
+
+            from google.cloud import debugger_v2
+
+            def sample_get_breakpoint():
+                # Create a client
+                client = debugger_v2.Debugger2Client()
+
+                # Initialize request argument(s)
+                request = debugger_v2.GetBreakpointRequest(
+                    debuggee_id="debuggee_id_value",
+                    breakpoint_id="breakpoint_id_value",
+                    client_version="client_version_value",
+                )
+
+                # Make the request
+                response = client.get_breakpoint(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.debugger_v2.types.GetBreakpointRequest, dict]):
                 The request object. Request to get breakpoint
@@ -303,7 +383,7 @@ class Debugger2AsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([debuggee_id, breakpoint_id, client_version])
         if request is not None and has_flattened_params:
@@ -360,6 +440,24 @@ class Debugger2AsyncClient:
     ) -> None:
         r"""Deletes the breakpoint from the debuggee.
 
+        .. code-block:: python
+
+            from google.cloud import debugger_v2
+
+            def sample_delete_breakpoint():
+                # Create a client
+                client = debugger_v2.Debugger2Client()
+
+                # Initialize request argument(s)
+                request = debugger_v2.DeleteBreakpointRequest(
+                    debuggee_id="debuggee_id_value",
+                    breakpoint_id="breakpoint_id_value",
+                    client_version="client_version_value",
+                )
+
+                # Make the request
+                client.delete_breakpoint(request=request)
+
         Args:
             request (Union[google.cloud.debugger_v2.types.DeleteBreakpointRequest, dict]):
                 The request object. Request to delete a breakpoint.
@@ -392,7 +490,7 @@ class Debugger2AsyncClient:
                 sent along with the request as metadata.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([debuggee_id, breakpoint_id, client_version])
         if request is not None and has_flattened_params:
@@ -447,6 +545,26 @@ class Debugger2AsyncClient:
     ) -> debugger.ListBreakpointsResponse:
         r"""Lists all breakpoints for the debuggee.
 
+        .. code-block:: python
+
+            from google.cloud import debugger_v2
+
+            def sample_list_breakpoints():
+                # Create a client
+                client = debugger_v2.Debugger2Client()
+
+                # Initialize request argument(s)
+                request = debugger_v2.ListBreakpointsRequest(
+                    debuggee_id="debuggee_id_value",
+                    client_version="client_version_value",
+                )
+
+                # Make the request
+                response = client.list_breakpoints(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.debugger_v2.types.ListBreakpointsRequest, dict]):
                 The request object. Request to list breakpoints.
@@ -476,7 +594,7 @@ class Debugger2AsyncClient:
                 Response for listing breakpoints.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([debuggee_id, client_version])
         if request is not None and has_flattened_params:
@@ -530,6 +648,26 @@ class Debugger2AsyncClient:
     ) -> debugger.ListDebuggeesResponse:
         r"""Lists all the debuggees that the user has access to.
 
+        .. code-block:: python
+
+            from google.cloud import debugger_v2
+
+            def sample_list_debuggees():
+                # Create a client
+                client = debugger_v2.Debugger2Client()
+
+                # Initialize request argument(s)
+                request = debugger_v2.ListDebuggeesRequest(
+                    project="project_value",
+                    client_version="client_version_value",
+                )
+
+                # Make the request
+                response = client.list_debuggees(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.debugger_v2.types.ListDebuggeesRequest, dict]):
                 The request object. Request to list debuggees.
@@ -559,7 +697,7 @@ class Debugger2AsyncClient:
                 Response for listing debuggees.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([project, client_version])
         if request is not None and has_flattened_params:
