@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -235,20 +235,20 @@ def test_debugger2_client_client_options(client_class, transport_class, transpor
     # unsupported value.
     with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "Unsupported"}):
         with pytest.raises(MutualTLSChannelError):
-            client = client_class()
+            client = client_class(transport=transport_name)
 
     # Check the case GOOGLE_API_USE_CLIENT_CERTIFICATE has unsupported value.
     with mock.patch.dict(
         os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "Unsupported"}
     ):
         with pytest.raises(ValueError):
-            client = client_class()
+            client = client_class(transport=transport_name)
 
     # Check the case quota_project_id is provided
     options = client_options.ClientOptions(quota_project_id="octopus")
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(transport=transport_name, client_options=options)
+        client = client_class(client_options=options, transport=transport_name)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file=None,
@@ -305,7 +305,7 @@ def test_debugger2_client_mtls_env_auto(
         )
         with mock.patch.object(transport_class, "__init__") as patched:
             patched.return_value = None
-            client = client_class(transport=transport_name, client_options=options)
+            client = client_class(client_options=options, transport=transport_name)
 
             if use_client_cert_env == "false":
                 expected_client_cert_source = None
@@ -382,6 +382,83 @@ def test_debugger2_client_mtls_env_auto(
                 )
 
 
+@pytest.mark.parametrize("client_class", [Debugger2Client, Debugger2AsyncClient])
+@mock.patch.object(
+    Debugger2Client, "DEFAULT_ENDPOINT", modify_default_endpoint(Debugger2Client)
+)
+@mock.patch.object(
+    Debugger2AsyncClient,
+    "DEFAULT_ENDPOINT",
+    modify_default_endpoint(Debugger2AsyncClient),
+)
+def test_debugger2_client_get_mtls_endpoint_and_cert_source(client_class):
+    mock_client_cert_source = mock.Mock()
+
+    # Test the case GOOGLE_API_USE_CLIENT_CERTIFICATE is "true".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
+        mock_api_endpoint = "foo"
+        options = client_options.ClientOptions(
+            client_cert_source=mock_client_cert_source, api_endpoint=mock_api_endpoint
+        )
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source(
+            options
+        )
+        assert api_endpoint == mock_api_endpoint
+        assert cert_source == mock_client_cert_source
+
+    # Test the case GOOGLE_API_USE_CLIENT_CERTIFICATE is "false".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "false"}):
+        mock_client_cert_source = mock.Mock()
+        mock_api_endpoint = "foo"
+        options = client_options.ClientOptions(
+            client_cert_source=mock_client_cert_source, api_endpoint=mock_api_endpoint
+        )
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source(
+            options
+        )
+        assert api_endpoint == mock_api_endpoint
+        assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "never".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
+        assert api_endpoint == client_class.DEFAULT_ENDPOINT
+        assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "always".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
+        assert api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT
+        assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "auto" and default cert doesn't exist.
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
+        with mock.patch(
+            "google.auth.transport.mtls.has_default_client_cert_source",
+            return_value=False,
+        ):
+            api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
+            assert api_endpoint == client_class.DEFAULT_ENDPOINT
+            assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "auto" and default cert exists.
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
+        with mock.patch(
+            "google.auth.transport.mtls.has_default_client_cert_source",
+            return_value=True,
+        ):
+            with mock.patch(
+                "google.auth.transport.mtls.default_client_cert_source",
+                return_value=mock_client_cert_source,
+            ):
+                (
+                    api_endpoint,
+                    cert_source,
+                ) = client_class.get_mtls_endpoint_and_cert_source()
+                assert api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT
+                assert cert_source == mock_client_cert_source
+
+
 @pytest.mark.parametrize(
     "client_class,transport_class,transport_name",
     [
@@ -400,7 +477,7 @@ def test_debugger2_client_client_options_scopes(
     options = client_options.ClientOptions(scopes=["1", "2"],)
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(transport=transport_name, client_options=options)
+        client = client_class(client_options=options, transport=transport_name)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file=None,
@@ -414,24 +491,26 @@ def test_debugger2_client_client_options_scopes(
 
 
 @pytest.mark.parametrize(
-    "client_class,transport_class,transport_name",
+    "client_class,transport_class,transport_name,grpc_helpers",
     [
-        (Debugger2Client, transports.Debugger2GrpcTransport, "grpc"),
+        (Debugger2Client, transports.Debugger2GrpcTransport, "grpc", grpc_helpers),
         (
             Debugger2AsyncClient,
             transports.Debugger2GrpcAsyncIOTransport,
             "grpc_asyncio",
+            grpc_helpers_async,
         ),
     ],
 )
 def test_debugger2_client_client_options_credentials_file(
-    client_class, transport_class, transport_name
+    client_class, transport_class, transport_name, grpc_helpers
 ):
     # Check the case credentials file is provided.
     options = client_options.ClientOptions(credentials_file="credentials.json")
+
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(transport=transport_name, client_options=options)
+        client = client_class(client_options=options, transport=transport_name)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file="credentials.json",
@@ -462,9 +541,72 @@ def test_debugger2_client_client_options_from_dict():
         )
 
 
-def test_set_breakpoint(
-    transport: str = "grpc", request_type=debugger.SetBreakpointRequest
+@pytest.mark.parametrize(
+    "client_class,transport_class,transport_name,grpc_helpers",
+    [
+        (Debugger2Client, transports.Debugger2GrpcTransport, "grpc", grpc_helpers),
+        (
+            Debugger2AsyncClient,
+            transports.Debugger2GrpcAsyncIOTransport,
+            "grpc_asyncio",
+            grpc_helpers_async,
+        ),
+    ],
+)
+def test_debugger2_client_create_channel_credentials_file(
+    client_class, transport_class, transport_name, grpc_helpers
 ):
+    # Check the case credentials file is provided.
+    options = client_options.ClientOptions(credentials_file="credentials.json")
+
+    with mock.patch.object(transport_class, "__init__") as patched:
+        patched.return_value = None
+        client = client_class(client_options=options, transport=transport_name)
+        patched.assert_called_once_with(
+            credentials=None,
+            credentials_file="credentials.json",
+            host=client.DEFAULT_ENDPOINT,
+            scopes=None,
+            client_cert_source_for_mtls=None,
+            quota_project_id=None,
+            client_info=transports.base.DEFAULT_CLIENT_INFO,
+            always_use_jwt_access=True,
+        )
+
+    # test that the credentials from file are saved and used as the credentials.
+    with mock.patch.object(
+        google.auth, "load_credentials_from_file", autospec=True
+    ) as load_creds, mock.patch.object(
+        google.auth, "default", autospec=True
+    ) as adc, mock.patch.object(
+        grpc_helpers, "create_channel"
+    ) as create_channel:
+        creds = ga_credentials.AnonymousCredentials()
+        file_creds = ga_credentials.AnonymousCredentials()
+        load_creds.return_value = (file_creds, None)
+        adc.return_value = (creds, None)
+        client = client_class(client_options=options, transport=transport_name)
+        create_channel.assert_called_with(
+            "clouddebugger.googleapis.com:443",
+            credentials=file_creds,
+            credentials_file=None,
+            quota_project_id=None,
+            default_scopes=(
+                "https://www.googleapis.com/auth/cloud-platform",
+                "https://www.googleapis.com/auth/cloud_debugger",
+            ),
+            scopes=None,
+            default_host="clouddebugger.googleapis.com",
+            ssl_credentials=None,
+            options=[
+                ("grpc.max_send_message_length", -1),
+                ("grpc.max_receive_message_length", -1),
+            ],
+        )
+
+
+@pytest.mark.parametrize("request_type", [debugger.SetBreakpointRequest, dict,])
+def test_set_breakpoint(request_type, transport: str = "grpc"):
     client = Debugger2Client(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -488,10 +630,6 @@ def test_set_breakpoint(
     assert isinstance(response, debugger.SetBreakpointResponse)
 
 
-def test_set_breakpoint_from_dict():
-    test_set_breakpoint(request_type=dict)
-
-
 def test_set_breakpoint_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -560,9 +698,15 @@ def test_set_breakpoint_flattened():
         # request object values.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-        assert args[0].debuggee_id == "debuggee_id_value"
-        assert args[0].breakpoint_ == data.Breakpoint(id="id_value")
-        assert args[0].client_version == "client_version_value"
+        arg = args[0].debuggee_id
+        mock_val = "debuggee_id_value"
+        assert arg == mock_val
+        arg = args[0].breakpoint_
+        mock_val = data.Breakpoint(id="id_value")
+        assert arg == mock_val
+        arg = args[0].client_version
+        mock_val = "client_version_value"
+        assert arg == mock_val
 
 
 def test_set_breakpoint_flattened_error():
@@ -603,9 +747,15 @@ async def test_set_breakpoint_flattened_async():
         # request object values.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-        assert args[0].debuggee_id == "debuggee_id_value"
-        assert args[0].breakpoint_ == data.Breakpoint(id="id_value")
-        assert args[0].client_version == "client_version_value"
+        arg = args[0].debuggee_id
+        mock_val = "debuggee_id_value"
+        assert arg == mock_val
+        arg = args[0].breakpoint_
+        mock_val = data.Breakpoint(id="id_value")
+        assert arg == mock_val
+        arg = args[0].client_version
+        mock_val = "client_version_value"
+        assert arg == mock_val
 
 
 @pytest.mark.asyncio
@@ -623,9 +773,8 @@ async def test_set_breakpoint_flattened_error_async():
         )
 
 
-def test_get_breakpoint(
-    transport: str = "grpc", request_type=debugger.GetBreakpointRequest
-):
+@pytest.mark.parametrize("request_type", [debugger.GetBreakpointRequest, dict,])
+def test_get_breakpoint(request_type, transport: str = "grpc"):
     client = Debugger2Client(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -649,10 +798,6 @@ def test_get_breakpoint(
     assert isinstance(response, debugger.GetBreakpointResponse)
 
 
-def test_get_breakpoint_from_dict():
-    test_get_breakpoint(request_type=dict)
-
-
 def test_get_breakpoint_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -721,9 +866,15 @@ def test_get_breakpoint_flattened():
         # request object values.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-        assert args[0].debuggee_id == "debuggee_id_value"
-        assert args[0].breakpoint_id == "breakpoint_id_value"
-        assert args[0].client_version == "client_version_value"
+        arg = args[0].debuggee_id
+        mock_val = "debuggee_id_value"
+        assert arg == mock_val
+        arg = args[0].breakpoint_id
+        mock_val = "breakpoint_id_value"
+        assert arg == mock_val
+        arg = args[0].client_version
+        mock_val = "client_version_value"
+        assert arg == mock_val
 
 
 def test_get_breakpoint_flattened_error():
@@ -764,9 +915,15 @@ async def test_get_breakpoint_flattened_async():
         # request object values.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-        assert args[0].debuggee_id == "debuggee_id_value"
-        assert args[0].breakpoint_id == "breakpoint_id_value"
-        assert args[0].client_version == "client_version_value"
+        arg = args[0].debuggee_id
+        mock_val = "debuggee_id_value"
+        assert arg == mock_val
+        arg = args[0].breakpoint_id
+        mock_val = "breakpoint_id_value"
+        assert arg == mock_val
+        arg = args[0].client_version
+        mock_val = "client_version_value"
+        assert arg == mock_val
 
 
 @pytest.mark.asyncio
@@ -784,9 +941,8 @@ async def test_get_breakpoint_flattened_error_async():
         )
 
 
-def test_delete_breakpoint(
-    transport: str = "grpc", request_type=debugger.DeleteBreakpointRequest
-):
+@pytest.mark.parametrize("request_type", [debugger.DeleteBreakpointRequest, dict,])
+def test_delete_breakpoint(request_type, transport: str = "grpc"):
     client = Debugger2Client(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -812,10 +968,6 @@ def test_delete_breakpoint(
     assert response is None
 
 
-def test_delete_breakpoint_from_dict():
-    test_delete_breakpoint(request_type=dict)
-
-
 def test_delete_breakpoint_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -888,9 +1040,15 @@ def test_delete_breakpoint_flattened():
         # request object values.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-        assert args[0].debuggee_id == "debuggee_id_value"
-        assert args[0].breakpoint_id == "breakpoint_id_value"
-        assert args[0].client_version == "client_version_value"
+        arg = args[0].debuggee_id
+        mock_val = "debuggee_id_value"
+        assert arg == mock_val
+        arg = args[0].breakpoint_id
+        mock_val = "breakpoint_id_value"
+        assert arg == mock_val
+        arg = args[0].client_version
+        mock_val = "client_version_value"
+        assert arg == mock_val
 
 
 def test_delete_breakpoint_flattened_error():
@@ -931,9 +1089,15 @@ async def test_delete_breakpoint_flattened_async():
         # request object values.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-        assert args[0].debuggee_id == "debuggee_id_value"
-        assert args[0].breakpoint_id == "breakpoint_id_value"
-        assert args[0].client_version == "client_version_value"
+        arg = args[0].debuggee_id
+        mock_val = "debuggee_id_value"
+        assert arg == mock_val
+        arg = args[0].breakpoint_id
+        mock_val = "breakpoint_id_value"
+        assert arg == mock_val
+        arg = args[0].client_version
+        mock_val = "client_version_value"
+        assert arg == mock_val
 
 
 @pytest.mark.asyncio
@@ -951,9 +1115,8 @@ async def test_delete_breakpoint_flattened_error_async():
         )
 
 
-def test_list_breakpoints(
-    transport: str = "grpc", request_type=debugger.ListBreakpointsRequest
-):
+@pytest.mark.parametrize("request_type", [debugger.ListBreakpointsRequest, dict,])
+def test_list_breakpoints(request_type, transport: str = "grpc"):
     client = Debugger2Client(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -980,10 +1143,6 @@ def test_list_breakpoints(
     assert response.next_wait_token == "next_wait_token_value"
 
 
-def test_list_breakpoints_from_dict():
-    test_list_breakpoints(request_type=dict)
-
-
 def test_list_breakpoints_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -1051,8 +1210,12 @@ def test_list_breakpoints_flattened():
         # request object values.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-        assert args[0].debuggee_id == "debuggee_id_value"
-        assert args[0].client_version == "client_version_value"
+        arg = args[0].debuggee_id
+        mock_val = "debuggee_id_value"
+        assert arg == mock_val
+        arg = args[0].client_version
+        mock_val = "client_version_value"
+        assert arg == mock_val
 
 
 def test_list_breakpoints_flattened_error():
@@ -1090,8 +1253,12 @@ async def test_list_breakpoints_flattened_async():
         # request object values.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-        assert args[0].debuggee_id == "debuggee_id_value"
-        assert args[0].client_version == "client_version_value"
+        arg = args[0].debuggee_id
+        mock_val = "debuggee_id_value"
+        assert arg == mock_val
+        arg = args[0].client_version
+        mock_val = "client_version_value"
+        assert arg == mock_val
 
 
 @pytest.mark.asyncio
@@ -1108,9 +1275,8 @@ async def test_list_breakpoints_flattened_error_async():
         )
 
 
-def test_list_debuggees(
-    transport: str = "grpc", request_type=debugger.ListDebuggeesRequest
-):
+@pytest.mark.parametrize("request_type", [debugger.ListDebuggeesRequest, dict,])
+def test_list_debuggees(request_type, transport: str = "grpc"):
     client = Debugger2Client(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -1134,10 +1300,6 @@ def test_list_debuggees(
     assert isinstance(response, debugger.ListDebuggeesResponse)
 
 
-def test_list_debuggees_from_dict():
-    test_list_debuggees(request_type=dict)
-
-
 def test_list_debuggees_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -1204,8 +1366,12 @@ def test_list_debuggees_flattened():
         # request object values.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-        assert args[0].project == "project_value"
-        assert args[0].client_version == "client_version_value"
+        arg = args[0].project
+        mock_val = "project_value"
+        assert arg == mock_val
+        arg = args[0].client_version
+        mock_val = "client_version_value"
+        assert arg == mock_val
 
 
 def test_list_debuggees_flattened_error():
@@ -1243,8 +1409,12 @@ async def test_list_debuggees_flattened_async():
         # request object values.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-        assert args[0].project == "project_value"
-        assert args[0].client_version == "client_version_value"
+        arg = args[0].project
+        mock_val = "project_value"
+        assert arg == mock_val
+        arg = args[0].client_version
+        mock_val = "client_version_value"
+        assert arg == mock_val
 
 
 @pytest.mark.asyncio
@@ -1281,6 +1451,23 @@ def test_credentials_transport_error():
             transport=transport,
         )
 
+    # It is an error to provide an api_key and a transport instance.
+    transport = transports.Debugger2GrpcTransport(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
+    options = client_options.ClientOptions()
+    options.api_key = "api_key"
+    with pytest.raises(ValueError):
+        client = Debugger2Client(client_options=options, transport=transport,)
+
+    # It is an error to provide an api_key and a credential.
+    options = mock.Mock()
+    options.api_key = "api_key"
+    with pytest.raises(ValueError):
+        client = Debugger2Client(
+            client_options=options, credentials=ga_credentials.AnonymousCredentials()
+        )
+
     # It is an error to provide scopes and a transport instance.
     transport = transports.Debugger2GrpcTransport(
         credentials=ga_credentials.AnonymousCredentials(),
@@ -1750,7 +1937,7 @@ def test_parse_common_location_path():
     assert expected == actual
 
 
-def test_client_withDEFAULT_CLIENT_INFO():
+def test_client_with_default_client_info():
     client_info = gapic_v1.client_info.ClientInfo()
 
     with mock.patch.object(
@@ -1815,3 +2002,33 @@ def test_client_ctx():
             with client:
                 pass
             close.assert_called()
+
+
+@pytest.mark.parametrize(
+    "client_class,transport_class",
+    [
+        (Debugger2Client, transports.Debugger2GrpcTransport),
+        (Debugger2AsyncClient, transports.Debugger2GrpcAsyncIOTransport),
+    ],
+)
+def test_api_key_credentials(client_class, transport_class):
+    with mock.patch.object(
+        google.auth._default, "get_api_key_credentials", create=True
+    ) as get_api_key_credentials:
+        mock_cred = mock.Mock()
+        get_api_key_credentials.return_value = mock_cred
+        options = client_options.ClientOptions()
+        options.api_key = "api_key"
+        with mock.patch.object(transport_class, "__init__") as patched:
+            patched.return_value = None
+            client = client_class(client_options=options)
+            patched.assert_called_once_with(
+                credentials=mock_cred,
+                credentials_file=None,
+                host=client.DEFAULT_ENDPOINT,
+                scopes=None,
+                client_cert_source_for_mtls=None,
+                quota_project_id=None,
+                client_info=transports.base.DEFAULT_CLIENT_INFO,
+                always_use_jwt_access=True,
+            )
